const arch = process.arch;
const platfrom = process.platform;
const extract = require('extract-zip');
const request = require('request');
const fs = require('fs');
const path = require('path');

const DIR_NAME = path.normalize(path.join(__dirname, '../ssf/'));
const FILE_NAME = DIR_NAME + 'ssf.zip';

function download(url) {
    return new Promise((resolve, reject) => {
        // initiate the download
        try {
            console.log('Downloading ' + url + ' ...');
            const req = request.get(url)
                .on('response', res => {

                    // create file write stream
                    const fileStream = fs.createWriteStream(FILE_NAME);

                    // setup piping
                    res.pipe(fileStream);

                    res.on('end', () => resolve());
                    res.on('error', err => resolve(err));
                });
            req.on('error', err => reject(err));
        } catch (e) {
            reject(e);
        }
    });
}

function unpack() {
    console.log('Unpacking ' + FILE_NAME + '...');
    return new Promise((resolve, reject) => {
        extract(FILE_NAME, {dir: DIR_NAME, defaultFileMode: 508 /* 0774 */}, err => {
            console.log('Unpack done');
            // extraction is complete. make sure to handle the err
            err ? reject(err) : resolve();
        });
    });
}

function copyFileSync(source, target) {
    let targetFile = target;

    // if target is a directory a new file with the same name will be created
    if (fs.existsSync(target)) {
        if (fs.lstatSync(target).isDirectory()) {
            targetFile = path.join(target, path.basename(source));
        }
    }

    fs.writeFileSync(targetFile, fs.readFileSync(source));
}

function copyFolderSync(source, target) {
    let files = [];

    if (!fs.existsSync(target)) {
        fs.mkdirSync(target);
    }

    // copy
    if (fs.lstatSync(source).isDirectory()) {
        files = fs.readdirSync(source);
        files.forEach(file => {
            const curSource = path.join(source, file);
            if (!fs.lstatSync(curSource).isDirectory()) {
                copyFileSync(curSource, target);
            }
        });
    }
}

function moveFiles() {
    console.log('Copy files to target directory...');
    const dirs = fs.readdirSync(DIR_NAME);
    const dir = dirs.find(dir => dir.match(/^ssf-.+/));
    if (!dir) {
        console.error('Cannot find directory in ' + DIR_NAME);
    } else {
        const dirName = path.join(DIR_NAME, dir);
        copyFolderSync(dirName, DIR_NAME);
        delDir(dirName);
    }
}

function checkRights() {
    if (platfrom === 'linux') {
        try {
            const stats = fs.statSync(path.join(DIR_NAME, 'ssf'));
            if (!(stats.mode & 0x40)) { // owner execution
                const output = require('child_process').execSync(`chmod u+x ${path.join(DIR_NAME, '*')}`);
                console.log('Set execution mode for ssf: ' + output);
            }
        } catch (e) {
            console.warn('Cannot execute chmod: ' + e);
        }
    }
}

function delDir(dirPath) {
    let files;
    try {
        files = fs.readdirSync(dirPath);
    } catch (e) {
        return;
    }
    if (files.length) {
        files.forEach(file => {
            const filePath = path.join(dirPath, file);
            if (fs.statSync(filePath).isFile()) {
                fs.unlinkSync(filePath);
            } else {
                delDir(filePath);

            }
        });
    }

    fs.rmdirSync(dirPath);
}

const links = {
    win32_x32: 'https://github.com/securesocketfunneling/ssf/releases/download/3.0.0/ssf-win-i386-3.0.0.zip',
    win32_x64: 'https://github.com/securesocketfunneling/ssf/releases/download/3.0.0/ssf-win-x86_64-3.0.0.zip',
    linux_x32: 'https://github.com/securesocketfunneling/ssf/releases/download/3.0.0/ssf-linux-i386-3.0.0.zip',
    linux_x64: 'https://github.com/securesocketfunneling/ssf/releases/download/3.0.0/ssf-linux-x86_64-3.0.0.zip',
    linux_ia32: 'https://github.com/securesocketfunneling/ssf/releases/download/3.0.0/ssf-linux-i386-3.0.0.zip',
    linux_ia64: 'https://github.com/securesocketfunneling/ssf/releases/download/3.0.0/ssf-linux-x86_64-3.0.0.zip',
    linux_arm: 'https://github.com/securesocketfunneling/ssf/releases/download/3.0.0/ssf-arm-linux-gnueabihf-3.0.0.zip'
};

if (!fs.existsSync(DIR_NAME)) {
    fs.mkdirSync(DIR_NAME);
}

if (!fs.existsSync(path.join(DIR_NAME, 'certs'))) {
    fs.mkdirSync(path.join(DIR_NAME, 'certs'));
}
if (!fs.existsSync(path.join(DIR_NAME, 'certs/trusted'))) {
    fs.mkdirSync(path.join(DIR_NAME, 'certs/trusted'));
}
if (fs.existsSync(DIR_NAME + 'ssf') || fs.existsSync(DIR_NAME + 'ssf.exe')) {
    process.exit();
} else if (links[platfrom + '_' + arch]) {
    download(links[platfrom + '_' + arch])
        .then(() => unpack())
        .then(() => {
            fs.unlinkSync(FILE_NAME);
            moveFiles();
            checkRights();
            console.log('SSF was downloaded and unpacked successfully');
            process.exit();
        })
        .catch(err => {
            console.error('Cannot download SSF package: ' + err);
            process.exit();
        })
} else {
    console.error(`Unfortunately your architecture ${platfrom}_${arch} is not supported.`);
    console.error('You can manually build it as described here: https://github.com/securesocketfunneling/ssf/blob/develop/BUILD_UNIX.md');
    process.exit(0);
}